/********************************************************************************
 Author : CAC (China Applications Support Team) 

 Date :   Nov, 2012

 File name :   AD7760.c

 Description :	 Use the PPI of BF527 to get to communicate with AD7760

 Hardware plateform : 	ADSP-BF527 EZ-Kit Lite Ver1.5 and EVAL-AD7760/62EDZ RevG
********************************************************************************/

#include <cdefBF527.h>
#include <stdio.h>
#include <sys\exception.h>
#include <ccblkfn.h>
#include <sysreg.h>
#include <signal.h>
#include <services_types.h>

#include "AD7760.h"
#include "BF527Driver.h"

/*******************************************************************
*   Function:    delay_loop
*   Description: Delay for a certain number of cycles, it's about 85us
*******************************************************************/
void delay_loop(void)
{
    int i = 0;
	
    //it will take about 10 core cycle for each loop.
    for( i = 0; i < 0xFF; i++ )
    {
        asm("nop;");	//must use it, or the delay is about 7us
    }
}

/*******************************************************************
*   Function:    delay_loop
*   Description: Delay for a certain number of cycles, it's about 25us
*******************************************************************/
void delay_ad(void)
{
    int i = 0;
	
    //it will take about 10 core cycle for each loop.
    for( i = 0; i < 0x3F; i++ )
    {
        asm("nop;");	
    }
}

/*******************************************************************
*   Function:    AD7760_Reset
*   Description: Reset AD7760 by PG5 of BF527, must reset ADC when powered up.
*******************************************************************/
void AD7760_Reset(void)
{
	int i,j;
	
	for(j=0;j<3;j++)
	{
		for(i=0;i<100;i++)
		{
			delay_loop();
			
		}
	}
	
	*pPORTGIO_CLEAR	= PG5;		
	delay_loop();
	

	*pPORTGIO_SET	= PG5;

	//
	for(j=0;j<380;j++)
	{
		for(i=0;i<100;i++)
		{
			delay_loop();
			
		}
	}
	
}


/*******************************************************************
*   Function:    AD7760_Write
*   Description: write the wanted register, it's usually used after powered up for initialization.
*******************************************************************/
void AD7760_Write(unsigned short Reg, unsigned short Reg_Data)
{
	int i;
	
	*pPORTFIO_SET = Reg;
	*pPORTFIO_CLEAR &= ~Reg;
	
	delay_loop();
	
	*pPORTGIO_CLEAR = AD7760_CS;
	*pPORTGIO_SET = AD7760_RD_WR;
	//*pPORTGIO_CLEAR = AD7760_RD_WR;
	
	delay_loop();
	
	*pPORTGIO_SET = AD7760_CS;
	
	/*********************************/
	delay_loop();

	/*********************************/
	
	*pPORTFIO_SET = Reg_Data;
	*pPORTFIO_CLEAR &= ~Reg_Data;
	
	*pPORTGIO_CLEAR = AD7760_CS;
	*pPORTGIO_SET = AD7760_RD_WR;
	
	delay_loop();
	
	*pPORTGIO_SET = AD7760_CS;
	
	delay_loop();
}


/*******************************************************************
*   Function:    AD7760_Read
*   Description: read the wanted register, it's used for reading STATE, GAIN, OFFSET and OVERRANGE registers.
*******************************************************************/
unsigned short Ctrl1_setting;	//Ctrl1_setting is globle.
void AD7760_Read(unsigned short Reg, unsigned short *pReg_Data)
{
	int i;
	
	//writing the register address
	*pPORTFIO_SET = CTRL_REG1;
	*pPORTFIO_CLEAR &= ~CTRL_REG1;
	asm("nop;");
		
	*pPORTGIO_CLEAR = AD7760_CS;
	*pPORTGIO_SET = AD7760_RD_WR;
	asm("nop;");	
	*pPORTGIO_SET = AD7760_CS;
	
	
	/*************************************************************************/
	//writing the relevant bit of control register1 to read the wanted register
	*pPORTFIO_SET = (1<<(Reg+9))|Ctrl1_setting;
	*pPORTFIO_CLEAR &= ~((1<<(Reg+9))|Ctrl1_setting);
	asm("nop;");
	*pPORTGIO_CLEAR = AD7760_CS;
	*pPORTGIO_SET = AD7760_RD_WR;	
	asm("nop;");
	*pPORTGIO_SET = AD7760_CS;	
	
	/************************************************************************/
	//convert the PFx to input mode before reading the register content
	*pPORTFIO_CLEAR = 0xFFFF;
	*pPORTFIO_DIR = 0x0000;
	*pPORTFIO_INEN	= 0xFFFF;
	asm("nop;");
		
	*pPORTGIO_CLEAR = AD7760_RD_WR;
	asm("nop;");
	*pPORTGIO_CLEAR = AD7760_CS;
	delay_loop();		//Note: must add this delay

	pReg_Data[0] = *pPORTFIO;
	
	*pPORTGIO_SET = AD7760_CS;
	asm("nop;");
	*pPORTGIO_SET = AD7760_RD_WR;
}



/*******************************************************************
*   Function:    AD_Read
*   Description: read ADC data, don't seperate the STATE bits(low 8-bit) from the original data
*******************************************************************/
void AD_Read(unsigned int *pADBuf)
{
	unsigned short temp;
	
	asm("nop;");
	asm("nop;");
	asm("nop;");
	asm("nop;");
	asm("nop;");
	
	//read the first 16bit MSBs data
	*pPORTGIO_CLEAR = AD7760_RD_WR;
	asm("nop;");
	*pPORTGIO_CLEAR = AD7760_CS;

	delay_ad();
	
	temp = *pPORTFIO;
	
	*pPORTGIO_SET = AD7760_CS;
	asm("nop;");
	*pPORTGIO_SET = AD7760_RD_WR;
	
	//read the second 16bit LSBs data
	*pPORTGIO_CLEAR = AD7760_RD_WR;
	asm("nop;");
	*pPORTGIO_CLEAR = AD7760_CS;
	delay_ad();
	
	*pADBuf = (temp) <<16 | *pPORTFIO;
	
	*pPORTGIO_SET = AD7760_CS;
	asm("nop;");
	*pPORTGIO_SET = AD7760_RD_WR;
	
}


