/********************************************************************************
 Author : CAC (China Applications Support Team) 

 Date :   Nov, 2012

 File name :   BF527Driver.c

 Description :	 Use the GPIOs of BF527 to get to communicate with AD7760

 Hardware plateform : 	ADSP-BF527 EZ-Kit Lite Ver1.5 and EVAL-AD7760/62EDZ RevG
********************************************************************************/

#include <cdefBF527.h>
#include <stdio.h>
#include <sys\exception.h>
#include <ccblkfn.h>
#include <sysreg.h>
#include <signal.h>
#include <services_types.h>
#include <bfrom.h>

#include "AD7760.h"
#include "BF527Driver.h"



/*******************************************************************
*   Function:    IRQ_DRDY
*   Description: ISR for DRDY
*******************************************************************/
unsigned int Data=0;
EX_INTERRUPT_HANDLER(IRQ_DRDY)
{

	*pPORTGIO_CLEAR = PG6;
	
	AD_Read(&Data);
	
	asm("nop;");
}

/*******************************************************************
*   Function:    InitPLL
*   Description: initializes the PLL registers
*******************************************************************/
void InitPLL(unsigned int msel, unsigned int ssel)
{	
	*pSIC_IWR0 |= 0x1;			/* enable PLL wakeup */
	*pPLL_CTL = SET_MSEL(msel);	/* 25MHz Xtal x MSEL = CCLK		*/

	idle();

	*pPLL_DIV = SET_SSEL(ssel);	/* CCLK/SSEL = SCLK				*/
	ssync();
}

/*******************************************************************
*   Function:    InitGPIOx
*   Description: Configure Port registers needed to run this test
*******************************************************************/
void InitGPIOx(void)
{
	
	//enable PG6(/RESET) output mode
	*pPORTGIO_DIR		|= PG5;
	*pPORTGIO_SET		= PG5;	
	
	delay_ad();
		
	//enable PG11(CS) and PG12(/RD/WR) output mode
	*pPORTG_FER 		= 0x0000;
	*pPORTGIO_DIR		|= PG11|PG12;	
	*pPORTGIO_SET		= PG11|PG12;

	// Setting bit 12 enables PPICLK
	//*pPORTF_MUX = 0x1000;
	//ssync();
	
	// initialize PFx for data transfering in output mode, it should be set to input mode when reading the registers.
	*pPORTFIO_INEN	= 0x0000;
	*pPORTF_FER = 0x0000;		//set 0 to enable PF GPIO mode.
	*pPORTFIO_DIR |= 0xFFFF;	//enable PFx output mode.
	ssync();

}

/*******************************************************************
*   Function:    Init_DRDY_Interrupt
*   Description: Initialize DRDY interrupt, PG6 pin and IVG12/Interrupt A are used.
*******************************************************************/
void Init_DRDY_Interrupt(void)
{   	
	//convert the PFx to input mode before reading
	*pPORTFIO_CLEAR = 0xFFFF;
	*pPORTFIO_DIR = 0x0000;
	*pPORTFIO_INEN	= 0xFFFF;
	
	*pPORTGIO_DIR		&= ~PG6;	// enable PG6 input mode
	*pPORTGIO_INEN		|= PG6;
	*pPORTGIO_EDGE		= PG6;
	*pPORTGIO_BOTH		&= ~PG6;	//set rising or falling sensitively
	*pPORTGIO_POLAR		= PG6;
	*pPORTGIO_MASKA_SET = PG6;
	
	register_handler(ik_ivg12, IRQ_DRDY);	// assign ISRs to interrupt vectors
	 	
	*pILAT |= EVT_IVG12;			// clear pending IVG12 interrupts
	ssync();
	
	*pSIC_IMASK1 |= 0x100;			//enable interrupt
}


/*******The following PPI functions are not used teperarily*********/


/*******************************************************************
*   Function:    PPI_RxIsr
*   Description: ISR for data received from the PPI.
*******************************************************************/
EX_INTERRUPT_HANDLER(PPI_RxIsr)
{
	int tempReg = 0;

	tempReg = *pDMA0_IRQ_STATUS;

	*pDMA0_IRQ_STATUS |= DMA_DONE;

	if( tempReg & 0x1)
	{
	    DisablePPI_DMA();
	}

}

/*******************************************************************
*   Function:    InitPPI
*   Description: Initialize PPI 0
*******************************************************************/
void InitPPI(void)
{
	*pPPI_CONTROL = FLD_SEL | PACK_EN | DLEN_16 | PORT_DIR | 0x0C | 0x10;
	ssync();
    
    
    //The PPI is set to receive 525 lines for each frame
	*pPPI_FRAME = 525; // this is active plus verticle blanking line total
	ssync();
}


/*******************************************************************
*   Function:    InitPPIInterrupts
*   Description: Initialize interrupts
*******************************************************************/
void InitPPIInterrupts(void)
{
	register_handler(ik_ivg8, PPI_RxIsr);		// assign ISR to interrupt vector

	*pILAT |= EVT_IVG8;							// clear pending IVG8 interrupts
	ssync();
	*pSIC_IMASK0 |= 0x00008000; // all interrupts 0=disabled but DMA0 PPI interrupt enabled
	ssync();

}

/*******************************************************************
*   Function:    InitDMA
*   Description: Initialize DMA
*******************************************************************/
void InitDMA(void)
{
	// Target address of the DMA
	//*pDMA0_START_ADDR = (void*)COLORBAR_START;

	// Line_Length 32bit transfers will be executed
	*pDMA0_X_COUNT = 2;

	// The modifier is set to 4 because of the 32bit transfers
	*pDMA0_X_MODIFY = 0x2;

	// Frame_Length 32bit transfers will be executed
	*pDMA0_Y_COUNT = 2;

	// The modifier is set to 4 because of the 32bit transfers
	*pDMA0_Y_MODIFY = 0x2;

	// DMA Config: Enable DMA | Generate interrupt on completion | Memory write DMA |
	// 32-bit xfers | 2-D DMA | Discard DMA FIFO before start
	*pDMA0_CONFIG = DI_EN | WNR | WDSIZE_16 | DMA2D | SYNC;
	ssync();
}

/*******************************************************************
*   Function:    EnablePPI
*   Description: Enable DMA and PPI 0
*******************************************************************/
void EnablePPI_DMA(void)
{
	// enable transfers
    *pDMA0_CONFIG |= 0x1;
    ssync();
    
	*pPPI_CONTROL |= PORT_EN;
	ssync();	
}

/*******************************************************************
*   Function:    DisablePPI_DMA
*   Description: Disable DMA and PPI 0
*******************************************************************/
void DisablePPI_DMA(void)
{
	// disable transfers
	*pDMA0_CONFIG = 0;
	ssync();
	
	*pPPI_CONTROL = 0;
	ssync();
}

